package ru.dbotthepony.kstarbound

import it.unimi.dsi.fastutil.objects.Object2ObjectFunction
import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap
import ru.dbotthepony.kstarbound.defs.player.RecipeDefinition
import java.util.Collections

class RecipeRegistry {
	private val recipesInternal = ArrayList<RecipeDefinition>()
	private val group2recipesInternal = Object2ObjectOpenHashMap<String, ArrayList<RecipeDefinition>>()
	private val group2recipesBacking = Object2ObjectOpenHashMap<String, List<RecipeDefinition>>()
	private val output2recipesInternal = Object2ObjectOpenHashMap<String, ArrayList<RecipeDefinition>>()
	private val output2recipesBacking = Object2ObjectOpenHashMap<String, List<RecipeDefinition>>()
	private val input2recipesInternal = Object2ObjectOpenHashMap<String, ArrayList<RecipeDefinition>>()
	private val input2recipesBacking = Object2ObjectOpenHashMap<String, List<RecipeDefinition>>()

	val recipes: List<RecipeDefinition> = Collections.unmodifiableList(recipesInternal)
	val group2recipes: Map<String, List<RecipeDefinition>> = Collections.unmodifiableMap(group2recipesBacking)
	val output2recipes: Map<String, List<RecipeDefinition>> = Collections.unmodifiableMap(output2recipesBacking)
	val input2recipes: Map<String, List<RecipeDefinition>> = Collections.unmodifiableMap(input2recipesBacking)

	fun add(recipe: RecipeDefinition) {
		recipesInternal.add(recipe)

		for (group in recipe.groups) {
			group2recipesInternal.computeIfAbsent(group, Object2ObjectFunction { p ->
				ArrayList<RecipeDefinition>().also {
					group2recipesBacking[p as String] = Collections.unmodifiableList(it)
				}
			}).add(recipe)
		}

		output2recipesInternal.computeIfAbsent(recipe.output.item.name, Object2ObjectFunction { p ->
			ArrayList<RecipeDefinition>().also {
				output2recipesBacking[p as String] = Collections.unmodifiableList(it)
			}
		}).add(recipe)

		for (input in recipe.input) {
			input2recipesInternal.computeIfAbsent(input.item.name, Object2ObjectFunction { p ->
				ArrayList<RecipeDefinition>().also {
					input2recipesBacking[p as String] = Collections.unmodifiableList(it)
				}
			}).add(recipe)
		}
	}
}
